##############################################
# Display error and close procedure 
##############################################

# Display error and close procedure
proc DisplayErrorAndClose {socketID code APIName} {
    global tcl_argv
    if {$code != -2} {
        set code2 [catch "ErrorStringGet $socketID $code strError"]
        if {$code2 != 0} {
            puts "$APIName ERROR => $code - ErrorStringGet ERROR => $code2"
            set tcl_argv(0) "$APIName ERROR => $code"
        } else {
            puts stdout "$APIName ERROR => $code : $strError"
            set tcl_argv(0) "$APIName ERROR => $code : $strError"
        }
    } else {
        puts stdout "$APIName ERROR => $code : TCP timeout"
        set tcl_argv(0) "$APIName ERROR => $code : TCP timeout"
    }
    set code2 [catch "TCP_CloseSocket $socketID"] 
    return
}



##################################################
# Main process
##################################################

# Initialization
set TimeOut 60
set Group "SINGLE_AXIS"
set Positioner "SINGLE_AXIS.MY_STAGE"
set BacklashValue 0.1
set Displacement 10
set code 0

# Open TCP socket
set code [catch "OpenConnection $TimeOut socketID"]
if {$code != 0} {
    puts stdout "OpenConnection failed => $code" 
} else {

    # Kill group    
    set code [catch "GroupKill $socketID $Group"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupKill" 
        return
    }

    # Enable backlach compenation
    # Caution : group must be not initialized and Backlash > 0 in the file stages.ini
    set code [catch "PositionerBacklashEnable $socketID $Positioner"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "PositionerBacklashEnable" 
        return
    }

    # Initialize group
    set code [catch "GroupInitialize $socketID $Group"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupInitialize" 
        return
    }

    # Home group
    set code [catch "GroupHomeSearch $socketID $Group"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupHomeSearch" 
        return
    }

    # Modify Backlash value
    # Caution : Backlash > 0 in the file stages.ini 
    set code [catch "PositionerBacklashSet $socketID $Positioner $BacklashValue"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "PositionerBacklashSet" 
        return
    }

    # Move group in positive direction
    set code [catch "GroupMoveRelative $socketID $Group $Displacement"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupMoveRelative" 
        return
    }

    # Move group in negative direction
    set code [catch "GroupMoveRelative $socketID $Group -$Displacement"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupMoveRelative" 
        return
    }

    # Disable Backlash (if you want to do trajectory, jogging or tracking)
    # Caution : to enable backlash, you must call GroupKill or KillAll to 
    # come back in not initialized status     
    set code [catch "PositionerBacklashDisable $socketID $Positioner"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "PositionerBacklashDisable" 
        return
    }

    # Close TCP socket
    set code [catch "TCP_CloseSocket $socketID"]
}
