##############################################
# Display error and close procedure 
##############################################

# Display error and close procedure
proc DisplayErrorAndClose {socketID code APIName} {
    global tcl_argv
    if {$code != -2} {
        set code2 [catch "ErrorStringGet $socketID $code strError"]
        if {$code2 != 0} {
            puts "$APIName ERROR => $code - ErrorStringGet ERROR => $code2"
            set tcl_argv(0) "$APIName ERROR => $code"
        } else {
            puts stdout "$APIName ERROR => $code : $strError"
            set tcl_argv(0) "$APIName ERROR => $code : $strError"
        }
    } else {
        puts stdout "$APIName ERROR => $code : TCP timeout"
        set tcl_argv(0) "$APIName ERROR => $code : TCP timeout"
    }
    set code2 [catch "TCP_CloseSocket $socketID"] 
    return
}



##################################################
# Main process
##################################################

# Initialization
set TimeOut 10
set group "alignstation"
set axis1 "alignstation.middle"
set axis2 "alignstation.base"
set digitalin "GPIO1.DI"

# Open TCP socket
set code [catch "OpenConnection $TimeOut socketID"]
if {$code != 0} {
    puts stdout "OpenConnection failed => $code" 
} else {

    # Kill group
    set code [catch "GroupKill $socketID $group"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupKill" 
        return
    }

    # Initialize group
    set code [catch "GroupInitialize $socketID $group"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupInitialize" 
        return
    }
      
    # Home group
    set code [catch "GroupHomeSearch $socketID $group"]
    if {$code != 0} {     
        DisplayErrorAndClose $socketID $code "GroupHomeSearch" 
        return
    }	

    # Move group with 5 relative units
    for { set var 0 } { $var <= 5 } { incr var } {
        set code [catch "GroupMoveRelative $socketID $group 1 1"]
        if {$code != 0} {	
            DisplayErrorAndClose $socketID $code "GroupMoveRelative" 
            return
        }
    }

    # Get digital value
    set code [catch "GPIODigitalGet $socketID $digitalin value"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GPIODigitalGet" 
        return
    }

    # Test if value of GPIO1.DI is different from 255
    while { $value != 255 } { 

        # Get digital value for the next iteration
        set code [catch "GPIODigitalGet $socketID $digitalin value"]
        if {$code != 0} {
            DisplayErrorAndClose $socketID $code "GPIODigitalGet" 
            return
        }
        
        puts stdout "$digitalin: $value"
        if { $value == 1 } {
            puts "           move axis1: 1"
            puts "           move axis2: -1"
    
            # Move axis 1
            set code [catch "GroupMoveAbsolute $socketID $axis1 1"]
            if {$code != 0} {
                DisplayErrorAndClose $socketID $code "GroupMoveAbsolute" 
                return
            }
    
            # Move axis 2
            set code [catch "GroupMoveAbsolute $socketID $axis2 -1"]
            if {$code != 0} {
                DisplayErrorAndClose $socketID $code "GroupMoveAbsolute" 
                return
            }
        } else {
            after 100
        }
        after 1000
    
    }
   
    # Wait 1 second and close socket
    after 1000
    puts "End of program"
    TCP_CloseSocket $socketID
}
