##############################################
# Display error and close procedure 
##############################################

# Display error and close procedure
proc DisplayErrorAndClose {socketID code APIName} {
    global tcl_argv
    if {$code != -2} {
        set code2 [catch "ErrorStringGet $socketID $code strError"]
        if {$code2 != 0} {
            puts "$APIName ERROR => $code - ErrorStringGet ERROR => $code2"
            set tcl_argv(0) "$APIName ERROR => $code"
        } else {
            puts stdout "$APIName ERROR => $code : $strError"
            set tcl_argv(0) "$APIName ERROR => $code : $strError"
        }
    } else {
        puts stdout "$APIName ERROR => $code : TCP timeout"
        set tcl_argv(0) "$APIName ERROR => $code : TCP timeout"
    }
    set code2 [catch "TCP_CloseSocket $socketID"] 
    return
}



##################################################
# Main process
##################################################

# Initialization
set TimeOut 60
set SlaveGroup "SINGLE_AXIS"
set XYGroup "XY"
set MasterPositioner "XY.X"
set MasterRatio 1
set code 0
set Displacement 65

# Open TCP socket
set code [catch "OpenConnection $TimeOut socketID"]
if {$code != 0} {
    puts stdout "OpenConnection failed => $code" 
} else {

    # Kill single axis group    
    set code [catch "GroupKill $socketID $SlaveGroup"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "Single axis GroupKill" 
        return
    }

    # Initialize single axis group
    set code [catch "GroupInitialize $socketID $SlaveGroup"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "Single axis GroupInitialize" 
        return
    }

    # Home single axis group
    set code [catch "GroupHomeSearch $socketID $SlaveGroup"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "Single axis GroupHomeSearch" 
        return
    }

    # Kill XY group    
    set code [catch "GroupKill $socketID $XYGroup"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "XY GroupKill" 
        return
    }

    # Initialize XY group
    set code [catch "GroupInitialize $socketID $XYGroup"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "XY GroupInitialize" 
        return
    }

    # Home XY group
    set code [catch "GroupHomeSearch $socketID $XYGroup"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "XY GroupHomeSearch" 
        return
    }

    # Set slave (single axis group) with its master (positioner from any group : XY here)
    set code [catch "SingleAxisSlaveParametersSet $socketID $SlaveGroup $MasterPositioner $MasterRatio"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "SingleAxisSlaveParametersSet" 
        return
    }

    # Enable master-slave mode (group must be ready)
    set code [catch "SingleAxisSlaveModeEnable $socketID $SlaveGroup"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "SingleAxisSlaveModeEnable" 
        return
    }

    # Move master positioner (the slave must follow the master in relation to a ratio)
    set code [catch "GroupMoveRelative $socketID $MasterPositioner $Displacement"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupMoveRelative" 
        return
    }

    # Disable master-slave mode
    set code [catch "SingleAxisSlaveModeDisable $socketID $SlaveGroup"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "SingleAxisSlaveModeDisable" 
        return
    }

    # Close TCP socket
    set code [catch "TCP_CloseSocket $socketID"]
}
