##############################################
# Display error and close procedure 
##############################################

# Display error and close procedure
proc DisplayErrorAndClose {socketID code APIName} {
    global tcl_argv
    if {$code != -2} {
        set code2 [catch "ErrorStringGet $socketID $code strError"]
        if {$code2 != 0} {
            puts "$APIName ERROR => $code - ErrorStringGet ERROR => $code2"
            set tcl_argv(0) "$APIName ERROR => $code"
        } else {
            puts stdout "$APIName ERROR => $code : $strError"
            set tcl_argv(0) "$APIName ERROR => $code : $strError"
        }
    } else {
        puts stdout "$APIName ERROR => $code : TCP timeout"
        set tcl_argv(0) "$APIName ERROR => $code : TCP timeout"
    }
    set code2 [catch "TCP_CloseSocket $socketID"] 
    return
}



##################################################
# Main process
##################################################

# Initialization
set TimeOut 60
set Group "XY"
set Positioner "XY.X"
set AnalogInput "GPIO2.ADC1"
set Offset 0
set Scale 1
set Velocity 20
set Acceleration 80
set TrackingType "Position"
set code 0

# Open TCP socket
set code [catch "OpenConnection $TimeOut socketID"]
if {$code != 0} {
    puts stdout "OpenConnection failed => $code" 
} else {

    # Kill group    
    set code [catch "GroupKill $socketID $Group"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupKill" 
        return
    }

    # Initialize group
    set code [catch "GroupInitialize $socketID $Group"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupInitialize" 
        return
    }

    # Home group
    set code [catch "GroupHomeSearch $socketID $Group"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupHomeSearch" 
        return
    }

    # Set analog tracking parameters
    set code [catch "PositionerAnalogTrackingPositionParametersSet $socketID $Positioner $AnalogInput $Offset $Scale $Velocity $Acceleration"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "PositionerAnalogTrackingPositionParametersSet" 
        return
    }

    # Enable analog position tracking mode (group must be ready)
    set code [catch "GroupAnalogTrackingModeEnable $socketID $Group $TrackingType"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupAnalogTrackingModeEnable" 
        return
    }

    # Change the amplitude of GPIO2.ADC1 analog input during 20 seconds
	after 20000

    # Disable analog position tracking mode
    set code [catch "GroupAnalogTrackingModeDisable $socketID $Group"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GroupAnalogTrackingModeDisable" 
        return
    }

    # Close TCP socket
    set code [catch "TCP_CloseSocket $socketID"]
}
