##############################################
# Display error and close procedure 
##############################################

# Display error and close procedure
proc DisplayErrorAndClose {socketID code APIName} {
    global tcl_argv
    if {$code != -2} {
        set code2 [catch "ErrorStringGet $socketID $code strError"]
        if {$code2 != 0} {
            puts "$APIName ERROR => $code - ErrorStringGet ERROR => $code2"
            set tcl_argv(0) "$APIName ERROR => $code"
        } else {
            puts stdout "$APIName ERROR => $code : $strError"
            set tcl_argv(0) "$APIName ERROR => $code : $strError"
        }
    } else {
        puts stdout "$APIName ERROR => $code : TCP timeout"
        set tcl_argv(0) "$APIName ERROR => $code : TCP timeout"
    }
    set code2 [catch "TCP_CloseSocket $socketID"] 
    return
}


##################################################
# Main process
##################################################

# Initialization
set TimeOut 120
set output  "GPIO1.DO"
set input   "GPIO1.DI"

# Open TCP socket
set code [catch "OpenConnection $TimeOut socketID"]
if {$code != 0} {
	puts stdout "OpenConnection failed => $code" 
} else {

    # Set output of GPIO1 to 255
    set code [catch "GPIODigitalSet $socketID $output 255 255"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GPIODigitalSet" 
        return
    }

    # Get value of output of GPIO1 and store it in OA
    set code [catch "GPIODigitalGet $socketID $output OA"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GPIODigitalGet" 
 	    return
    } else {
        puts "OA = $OA"
    }

    # Set output of GPIO1 to 0
    set code [catch "GPIODigitalSet $socketID $output 255 0"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GPIODigitalSet" 
        return
    }

    # Get value of output of GPIO1 and store it in OB
    set code [catch "GPIODigitalGet $socketID $output OB"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GPIODigitalGet" 
        return
    } else {
        puts "OB = $OB"
    }

    # Set output of GPIO1 to 63 (mask value)
    set code [catch "GPIODigitalSet $socketID $output 63 255"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GPIODigitalSet" 
        return
    }

    # Get value of output of GPIO1 and store it in OC
    set code [catch "GPIODigitalGet $socketID $output OC"]
    if {$code != 0} {
        DisplayErrorAndClose $socketID $code "GPIODigitalGet" 
        return
    } else {
        puts "OC = $OC"
    }

    # Test if OA = 255 and OB = 0
    if {$OA == 255 & $OB == 0} {
        puts "Digital outputs OK"
        # Test if OC = 63
        if {$OC == 63} {
            puts "Mask OK"
        } else {
            puts "Pb Mask"
        }
    } else {
        puts "Pb digital outputs"
    }
    
    # Close socket
    TCP_CloseSocket $socketID
}
